@@ -584,12 +584,10 @@ sub libSuffix($) {
 #-----------------------------------------------------------------------------
     my $suffix;
 
-    if ($platform eq 'windows') {
-        $suffix = '.a';
-    } elsif ($platform eq 'darwin') {
-        $suffix = '.dylib';
+    if ($platform eq "windows") {
+        $suffix = ".a";
     } else {
-        $suffix = '.so';
+        $suffix = ".so";
     }
 }
 
@@ -1001,8 +999,8 @@ sub getX11Library($@) {
 
         if (-d('/usr/link/X11')) {
             $default = '/usr/link/X11/libX11' . libSuffix($platform);
-        } elsif (-d('/usr/X11R6')) {
-            $default = '/usr/X11R6/lib/libX11' . libSuffix($platform);
+        } elsif (-d('/usr/lib/X11')) {
+            $default = '/usr/lib/libX11' . libSuffix($platform);
         } else {
             $default = "libX11" . libSuffix($platform);
         }
@@ -1978,7 +1976,7 @@ if ($platform eq "GNU") {
     push(@Makefile_config, 'CFLAGS_SHLIB = -fno-common', "\n");
     push(@Makefile_config, "LDSHLIB = ",
          "-dynamiclib ",
-         '-install_name $(NETPBMLIB_RUNTIME_PATH)/libnetpbm.$(MAJ).dylib', 
+         '-install_name $(NETPBM_RUNTIME_PATH)/libnetpbm.$(MAJ).dylib', 
          "\n");
 #    push(@Makefile_config, "INSTALL = install\n");
 } else {
