@@ -311,19 +311,12 @@ execLdconfig() {
 
 
 
-sub ldconfigExists() {
-
-    return (system("ldconfig -? 2>/dev/null") >> 8) != 127;
-}
-
-
-
 sub
 doLdconfig() {
 #-----------------------------------------------------------------------------
 #  Run Ldconfig where appropriate.
 #-----------------------------------------------------------------------------
-    if ($OSNAME eq "linux" || ldconfigExists()) {
+    if ($OSNAME eq "linux" || system("ldconfig -? 2>/dev/null") != 127) {
         # This is a system where Ldconfig makes sense
 
         print("In order for the Netpbm shared library to be found when " .
@@ -829,7 +822,7 @@ installConfig($$$$$$$) {
 
     my $configTemplateFilename = dirname($0) . "/config_template";
 
-    my $templateOpened = open(TEMPLATE, "<$configTemplateFilename");
+    my $templateOpened = open(TEMPLATE, "<", $configTemplateFilename);
     if (!$templateOpened) {
         $error = "Can't open template file '$configTemplateFilename'.\n";
     } else {
@@ -837,7 +830,7 @@ installConfig($$$$$$$) {
 
         close(TEMPLATE);
 
-        my $versionOpened = open(VERSION, "<$pkgdir/VERSION");
+        my $versionOpened = open(VERSION, "<", "$pkgdir/VERSION");
 
         my $version;
         if (!$versionOpened) {
@@ -859,7 +852,7 @@ installConfig($$$$$$$) {
             
             my $filename = "$bindir/netpbm-config";
             
-            my $success = open(NETPBM_CONFIG, ">$filename");
+            my $success = open(NETPBM_CONFIG, ">", $filename);
             if ($success) {
                 chmod(0755, $filename);
                 foreach (@{$fileContentsR}) { print NETPBM_CONFIG; }
