@@ -90,7 +90,7 @@ validateComputableSize(struct pam * const pamP) {
 
     if (depth > INT_MAX/sizeof(sample))
         pm_error("image depth (%u) too large to be processed", depth);
-    else if (pamP->width > 0 && depth * sizeof(sample) > INT_MAX/pamP->width)
+    else if (depth * sizeof(sample) > INT_MAX/pamP->width)
         pm_error("image width and depth (%u, %u) too large "
                  "to be processed.", pamP->width, depth);
     else if (pamP->width * (depth * sizeof(sample)) >
@@ -679,9 +679,9 @@ pnm_readpaminitrestaspnm(FILE * const fileP,
 -----------------------------------------------------------------------------*/
     struct pam pam;
 
-    pam.size        = sizeof(struct pam);
+    pam.size        = PAM_STRUCT_SIZE(tuple_type);
     pam.file        = fileP;
-    pam.len         = PAM_STRUCT_SIZE(tuple_type);
+    pam.len         = sizeof(struct pam);
     pam.format      = PAM_FORMAT;
 
     readpaminitrest(&pam);
